# Profiling #

If the server is started with the `run-debug-single` make target, it will expose
the JMX interface, by default on port 9090.  Using a profiler such as VisualVM,
one can then connect to the CaosDB server and profile execution times.

## Example settings for VisualVM ##

In the sampler settings, you may want to add these expressions to the blocked
packages: `org.restlet.**, com.mysql.**`.  Branches on the call tree which are
entirely inside the blacklist, will become leaves.  Alternatively, specify a
whitelist, for example with `org.caosdb.server.database.backend.implementation.**`,
if you only want to see the time spent for certain MySQL calls.

# Manual Java-side benchmarking #

Benchmarking can be done using the `TransactionBenchmark` class (in package
`org.caosdb.server.database.misc`).

- Single timings can be added to instances of that class via the
  `addBenchmark(object, time)` method.  Multiple benchmarks for the same object
  (typically just strings) can be averaged.
- Benchmarks can be serialized into XML, `Container` and `Query` objects already
  use this with their included benchmarks to output benchmarking results.
- To work with the benchmarks of often used objects, use these methods:
  - `Container.getTransactionBenchmark().addBenchmark()`
  - `Query.addBenchmark()`

# Miscellaneous notes #

Notes to self, details, etc.

## On method calling order and benchmarked events ##

- `Transaction.execute()` :: Logs benchmarks for events like:
  - `INIT` :: The transaction's `init()` method.
  - `PRE_CHECK`
  - `CHECK`
  - `POST_CHECK`
  - `PRE_TRANSACTION`
  - `TRANSACTION` -> typically calls
    `database.backend.transaction.[BackendTransaction].execute()`, which in turn
    calls, some levels deeper, `backend.transaction.....execute(<k extends
    BackendTransaction> t)` -> see next point
  - ...
- `backend.transaction.[...].execute(transaction)` :: This method is benchmarked
  again (via parent class `BackendTransaction`), this is probably the deepest
  level of benchmarking currently (Benchmark is logged as
  e.g. `<RetrieveFullEntity>...</>`).  It finally calls
  `[MySQLTransaction].execute()`.
- `[MySQLTransaction].execute()` :: This is the deepest backend implementation
  part, it typically creates a prepared statement and executes it.
- Currently not benchmarked separately:
  - Getting the actual implementation (probably fast?)
  - Preparing the SQL statement
  - Executing the SQL statement
  - Java-side caching

## Server settings ##

- To enable the SQL general logs, log into the SQL server and do:
  ```sql
SET GLOBAL log_output = 'TABLE';
SET GLOBAL general_log = 'ON';
```
- To enable transaction benchmarks and disable caching in the server, set these
  server settings:
```conf
TRANSACTION_BENCHMARK_ENABLED=true
CACHE_DISABLE=true
```
- Additionally, the server should be started via `make run-debug` (instead of
  `make run-single`), otherwise the benchmarking will not be active.

## Notable benchmarks and where to find them ##

| Name                                 | Where measured                               | What measured                 |
|--------------------------------------|----------------------------------------------|-------------------------------|
| `Retrieve.init`                      | transaction/Transaction.java#135             | transaction/Retrieve.java#48  |
| `Retrieve.transaction`               | transaction/Transaction.java#174             | transaction/Retrieve.java#133 |
| `Retrieve.post_transaction`          | transaction/Transaction.java#182             | transaction/Retrieve.java#77  |
| `EntityResource.httpGetInChildClass` | resource/transaction/EntityResource.java#118 | all except XML generation     |
| `ExecuteQuery`                       | ?                                            | ?                             |
|                                      |                                              |                               |
